"""
Quick Nakdan https://nakdanlive.dicta.org.il
Pro Nakdan https://nakdanpro.dicta.org.il
Manual Nakdan https://www.yo-yoo.co.il/tools/nikud

uv run examples/phonemize.py
"""

from phonikud import phonemize

sentences = [
    # "הִיא אָמְרָה לִי",  # Hirik with Yod vowel
    # "מָה קוֹרֶה?",  # Vav with Holam vowel
    # "הוּא קֵרֵחַ",  # Het gnuva
    # "גִ'ירָפָה זַ'רְגּוֹן",  # Geresh
    # "שָׁלוֹם שׂוֹרֵר",  # Shin Sin
    # "לְבִיבָה יְלָדִים שְׁאֵלָה בְּהֵמָה זְעָקוֹת בְּרֹאשׁ גּוֹרְרִים מְצַפְצְפִים מִרְפְּאוֹתֵיהֶם",  # Shva na
    # "בְּרוֹשׁ סְבִיבָה בְּרָגִים שְׁפֵלָה בְּרֵכָה זְנָבוֹת גּוֹרְמִים הָלַכְתְּ וּבְרָכָה",  # Shva nax
    # "אַתָּה חַיָּב לִי 50 שֶׁקֶל וְהַסֵּמֶל הוּא ₪",  # numbers, symbols
    # "כל",  # dictionary
    # "אֶלֶגַנְטִי",  # lookup
    # "כָּל",  # match to one without nikud
    # "יִשְׂרָאֵל הוּקְמָה בִּשְׁנַת 1948",
    # "₪",
    # "מָחָר בְּשָׁעָה 22:00",  # time to word
    # "אוֹרוֹת ווֹלְטֶר ווֹלֶנְטַיְן אֻרְווֹת אוֹרְחִים אָרְבוּ הוֹוֶה וּוּבְּרִים וָו וִי וּוִיקְטוֹר",  # Vavs
    # "לְמַיִם אֱלֹהֵי",
    # "שְׁמַע אָח שֶׁלִּי, תִּרְאֶה. נִרְאֶה לִי שֶׁזֶּה עוֹבֵד!",
    # "גִ'ירָפָה",
    # "יִהְיֶה",
    # "בְּמָה",
    # "בְּיִשְׂרָאֵל.",
    # "אֲוִויר",
    # "לַכִּיווּן",
    # """קִיוּוּ""",
    # "אֻרְווֹת",
    # "וּוִיקְטוֹר",
    # "אֲוִויר",
    # "מְאוּוְרֶרֶת",
    # "טוֹוִים",
    # "מִגְוַון",
    # "הַקַּווְקָז",
    # "בְּסִיוֹוֹן",
    # "צוֹוַחַת",
    # "צוֹוְחִים",
    # "וּוֶנְדֶּר",
    # "וּוַעֲדַת",
    # "שָׁלוֹם",
    # "וּמֶה",
    # "הַתְּרוּמָה",
    # "English",
    # "בַּרִ֫יא",
    # "זֶ֫ה הָיָ֫ה קוֹרֵ֫עַ מִצְּח֫וֹק הֵ֫ם קוֹרְעִ֫ם אוֹתִ֫י",
    # "וּבֽתוֹלְד֫וֹת",
    # "וִירוּשָׁלַיִם",
    # "וְאַתָּה",
    # "אוּרִי",
    # "אַוָּרִי",
    # "וָואלָה",
    # "ווֹלְטֶר",
    # "אוֹרִי",
    # "אֻרִי",
    # "גִּ׳ירָהּ",
    # "גִּ'רַ֫פָה",
    # "זָ'קֵ֫ט",
    # "תַּ'מְבִין",
    # "כֹּחַ",
    # "מֹחַ",
    # "כָּל",
    # "רֵיחַ",
    # "אֶלֶף עֶרֶב",
    # "לִוְיָתָ֫ן",
    # "יִשָּׂשכָר",
    # "יִשָּׂשכָ֫ר",
    # "יַאלְלָה",
    # "אִיָּ֫ר",
    # "קוֹרֶה",
    # "אָח",
    # "שְׁמוּרִים",
    # "עֶ֫רֶב",
    # "מַגְנִיבָה",
    # "אוֹפַנַּיִים",
    # "אֵיתָנוּת",
    # "רֵיחַ",
    # "אֵיתָנוּת",
    # "אֵיר֫וֹפָּה",
    # "צָהֳרַ֫יִם",
    # "דַּוָָר",
    # "עַכְשָׁ֫יו",
    # "כׇּל אֶחָד יָכוֹל!",
    # "הַמָּלֵא",
    # "רוּחַ",
    # "אֶנְצִיקְלוֹפֶּ֫דְיָה",
    # 'צַהַ"ל',
    # "שָׁלוֹם",
    # "אֶנְצִיקְלוֹפֶּדְיָה",
    # "אוֹפַנַּייִם @"c
    # "קֵרֵחַ",
    # "שָׁלוֹם חֲבֵרִים! מָה שְׁלוֹמְכֶם?",
    # "אֶנְצִיקְלוֹפֶּ֫דְיָה",
    # "וַאלְלָה",
    # "בָּךְ",
    # "חֲתֻונָּה",
    # "דִּוּ֫וּחַ",
    # "שֶׁמְּֽשַׁוֶּ֫קֶת",
    # "וּמֽק֫וֹם",
    # "ו֫וֹלְטֶר",  # double vav start
    # "הֲו֫וֹלְטֶר",  # double vav middle
    # "וַואטְסְאַפּ"
    # "עַל-כֵּן",
    # "לְהַגְבִּיהַּ",
    # "לִשְׁקֹעַ",
    # "כׇּל",
    # "אֵיבָרָיו",
    # "סְתָיו",
    # "לֽאַט",
    # "רֽאִי",
    # "אֵלָיו",
    # "ו֫וֹלְטֶר",
    # "צַהַ״ל",
    # 'צַהַ"ל',
    # "לִרְאוֹת",
    # "סִי֯֫יַּמְתִּי",
    # "טְרַא֯מְפּ"
    "אֲוְורִירִי",
    "לִוְויְיתָן",
    "טַוָוס",
]
# print(get_syllables("אֵלָיו"))
for sentence in sentences:
    phonemes = phonemize(
        sentence,
        preserve_punctuation=True,
        use_post_normalize=True,
        use_expander=True,
        predict_stress=True,
        predict_shva_nah=True,
        stress_placement="vowel",
        schema="modern",
    )
    print(phonemes)
